@@ -43,7 +43,7 @@
 #include "camel-mapi-folder.h"
 #include "camel-mapi-store-summary.h"
 #include "camel-mapi-utils.h"
-#define d(x) x
+#define d(x)
 
 #include <exchange-mapi-defs.h>
 #include "exchange-mapi-mail-utils.h"
@@ -63,16 +63,6 @@ mapi_message_item_send (ExchangeMapiConnection *conn, MailItem *item, GError **p
 	guint64 fid = 0;
 	mapi_id_t mid = 0;
 
-	#define unset(x) g_free (x); x = NULL
-
-	item->header.flags = MSGFLAG_UNSENT;
-	unset (item->header.from);
-	unset (item->header.from_email);
-	unset (item->header.transport_headers);
-	item->header.recieved_time = 0;
-
-	#undef unset
-
 	mid = exchange_mapi_connection_create_item (conn, olFolderSentMail, fid,
 					 camel_mapi_utils_create_item_build_props, item,
 					 item->recipients, item->attachments, item->generic_streams, MAPI_OPTIONS_DELETE_ON_SUBMIT_FAILURE, perror);
@@ -81,8 +71,12 @@ mapi_message_item_send (ExchangeMapiConnection *conn, MailItem *item, GError **p
 }
 
 static gboolean
-mapi_send_to (CamelTransport *transport, CamelMimeMessage *message,
-	      CamelAddress *from, CamelAddress *recipients, GError **error)
+mapi_send_to_sync (CamelTransport *transport,
+                   CamelMimeMessage *message,
+                   CamelAddress *from,
+                   CamelAddress *recipients,
+                   GCancellable *cancellable,
+                   GError **error)
 {
 	ExchangeMapiConnection *conn;
 	MailItem *item = NULL;
@@ -111,7 +105,7 @@ mapi_send_to (CamelTransport *transport, CamelMimeMessage *message,
 	}
 
 	/* Convert MIME to MailItem, attacment lists and recipient list.*/
-	item = camel_mapi_utils_mime_to_item (message, 0, from, NULL);
+	item = camel_mapi_utils_mime_to_item (message, from, cancellable, NULL);
 
 	/* send */
 	st = mapi_message_item_send (conn, item, error);
@@ -158,7 +152,7 @@ camel_mapi_transport_class_init (CamelMapiTransportClass *class)
 	service_class->get_name = mapi_transport_get_name;
 
 	transport_class = CAMEL_TRANSPORT_CLASS (class);
-	transport_class->send_to = mapi_send_to;
+	transport_class->send_to_sync = mapi_send_to_sync;
 }
 
 static void
