@@ -45,7 +45,7 @@
 #include "camel-mapi-store-summary.h"
 #include "camel-mapi-utils.h"
 
-#define d(x) x
+#define d(x)
 
 #define STREAM_SIZE 4000
 
@@ -98,15 +98,12 @@ mail_item_add_recipient (const gchar *recipients, OlMailRecipientType type, GSLi
 }
 
 static void
-mail_item_set_from(MailItem *item, const gchar *from_name, const gchar *from_email)
+mail_item_set_from(MailItem *item, const gchar *from)
 {
 	if (item->header.from)
 		g_free (item->header.from);
-	if (item->header.from_email)
-		g_free (item->header.from_email);
 
-	item->header.from = g_strdup (from_name);
-	item->header.from_email = g_strdup (from_email);
+	item->header.from = g_strdup (from);
 }
 
 static void
@@ -121,7 +118,7 @@ mail_item_set_subject(MailItem *item, const gchar *subject)
 #define MAX_READ_SIZE 0x1000
 
 static void
-mail_item_set_body_stream (MailItem *item, CamelStream *body, MailItemPartType part_type)
+mail_item_set_body_stream (MailItem *item, CamelStream *body, MailItemPartType part_type, GCancellable *cancellable)
 {
 	guint8 *buf = g_new0 (guint8 , STREAM_SIZE);
 	guint32	read_size = 0, i;
@@ -132,7 +129,7 @@ mail_item_set_body_stream (MailItem *item, CamelStream *body, MailItemPartType p
 
 	stream->value = g_byte_array_new ();
 
-	while (read_size = camel_stream_read (body, (gchar *)buf, STREAM_SIZE, NULL), read_size > 0) {
+	while (read_size = camel_stream_read (body, (gchar *)buf, STREAM_SIZE, cancellable, NULL), read_size > 0) {
 		stream->value = g_byte_array_append (stream->value, buf, read_size);
 
 		is_null_terminated = buf [read_size - 1] == 0;
@@ -189,7 +186,7 @@ mail_item_set_body_stream (MailItem *item, CamelStream *body, MailItemPartType p
 }
 
 static gboolean
-mail_item_add_attach (MailItem *item, CamelMimePart *part, CamelStream *content_stream)
+mail_item_add_attach (MailItem *item, CamelMimePart *part, CamelStream *content_stream, GCancellable *cancellable)
 {
 	guint8 *buf = g_new0 (guint8 , STREAM_SIZE);
 	const gchar *content_id = NULL;
@@ -251,7 +248,7 @@ mail_item_add_attach (MailItem *item, CamelMimePart *part, CamelStream *content_
 	stream->value = g_byte_array_new ();
 
 	camel_seekable_stream_seek((CamelSeekableStream *)content_stream, 0, CAMEL_STREAM_SET, NULL);
-	while (read_size = camel_stream_read(content_stream, (gchar *)buf, STREAM_SIZE, NULL), read_size > 0) {
+	while (read_size = camel_stream_read(content_stream, (gchar *)buf, STREAM_SIZE, cancellable, NULL), read_size > 0) {
 		stream->value = g_byte_array_append (stream->value, buf, read_size);
 	}
 
@@ -262,7 +259,7 @@ mail_item_add_attach (MailItem *item, CamelMimePart *part, CamelStream *content_
 }
 
 static gboolean
-mapi_do_multipart (CamelMultipart *mp, MailItem *item, gboolean *is_first)
+mapi_do_multipart (CamelMultipart *mp, MailItem *item, gboolean *is_first, GCancellable *cancellable)
 {
 	CamelDataWrapper *dw;
 	CamelStream *content_stream;
@@ -283,7 +280,7 @@ mapi_do_multipart (CamelMultipart *mp, MailItem *item, gboolean *is_first)
 		dw = camel_medium_get_content (CAMEL_MEDIUM (part));
 		if (CAMEL_IS_MULTIPART(dw)) {
 			/* recursive */
-			if (!mapi_do_multipart (CAMEL_MULTIPART (dw), item, is_first))
+			if (!mapi_do_multipart (CAMEL_MULTIPART (dw), item, is_first, cancellable))
 				return FALSE;
 			continue;
 		}
@@ -291,7 +288,8 @@ mapi_do_multipart (CamelMultipart *mp, MailItem *item, gboolean *is_first)
 		filename = camel_mime_part_get_filename(part);
 
 		content_stream = camel_stream_mem_new();
-		content_size = camel_data_wrapper_decode_to_stream (dw, (CamelStream *) content_stream, NULL);
+		content_size = camel_data_wrapper_decode_to_stream_sync (
+			dw, (CamelStream *) content_stream, cancellable, NULL);
 
 		camel_seekable_stream_seek((CamelSeekableStream *)content_stream, 0, CAMEL_STREAM_SET, NULL);
 
@@ -301,33 +299,20 @@ mapi_do_multipart (CamelMultipart *mp, MailItem *item, gboolean *is_first)
 		type = camel_mime_part_get_content_type(part);
 
 		if (i_part == 0 && (*is_first) && camel_content_type_is (type, "text", "plain")) {
-			mail_item_set_body_stream (item, content_stream, PART_TYPE_PLAIN_TEXT);
+			mail_item_set_body_stream (item, content_stream, PART_TYPE_PLAIN_TEXT, cancellable);
 			*is_first = FALSE;
 		} else if (camel_content_type_is (type, "text", "html")) {
-			mail_item_set_body_stream (item, content_stream, PART_TYPE_TEXT_HTML);
+			mail_item_set_body_stream (item, content_stream, PART_TYPE_TEXT_HTML, cancellable);
 		} else {
-			mail_item_add_attach (item, part, content_stream);
+			mail_item_add_attach (item, part, content_stream, cancellable);
 		}
 	}
 
 	return TRUE;
 }
 
-static void
-mail_item_set_time (time_t *item_time, time_t camel_time, gint camel_offset)
-{
-	if (camel_time == CAMEL_MESSAGE_DATE_CURRENT) {
-		*item_time = 0;
-	} else {
-		/* Convert to UTC */
-		/*camel_time -= (camel_offset / 100) * 60 * 60;
-		camel_time -= (camel_offset % 100) * 60;*/
-		*item_time = camel_time;
-	}
-}
-
 MailItem *
-camel_mapi_utils_mime_to_item (CamelMimeMessage *message, gint32 message_camel_flags, CamelAddress *from, GError **error)
+camel_mapi_utils_mime_to_item (CamelMimeMessage *message, CamelAddress *from, GCancellable *cancellable, GError **error)
 {
 	CamelDataWrapper *dw = NULL;
 	CamelContentType *type;
@@ -335,12 +320,9 @@ camel_mapi_utils_mime_to_item (CamelMimeMessage *message, gint32 message_camel_f
 	CamelMultipart *multipart;
 	CamelInternetAddress *to, *cc, *bcc;
 	MailItem *item = g_new0 (MailItem, 1);
-	const gchar *namep = NULL;
-	const gchar *addressp = NULL;
+	const gchar *namep;
+	const gchar *addressp;
 	const gchar *content_type;
-	time_t msg_time = 0;
-	gint msg_time_offset = 0;
-	GArray *headers;
 
 	gssize	content_size;
 	GSList *recipient_list = NULL;
@@ -359,18 +341,7 @@ camel_mapi_utils_mime_to_item (CamelMimeMessage *message, gint32 message_camel_f
 		namep = NULL;
 	}
 
-	item->header.flags = 0;
-	if (message_camel_flags & CAMEL_MESSAGE_SEEN)
-		item->header.flags |= MSGFLAG_READ;
-	if (message_camel_flags & CAMEL_MESSAGE_ATTACHMENTS)
-		item->header.flags |= MSGFLAG_HASATTACH;
-
-	mail_item_set_from (item, namep, addressp);
-
-	msg_time = camel_mime_message_get_date (message, &msg_time_offset);
-	if (msg_time == CAMEL_MESSAGE_DATE_CURRENT)
-		msg_time = camel_mime_message_get_date_received (message, &msg_time_offset);
-	mail_item_set_time (&item->header.recieved_time, msg_time, msg_time_offset);
+	mail_item_set_from (item, namep);
 
 	to = camel_mime_message_get_recipients(message, CAMEL_RECIPIENT_TYPE_TO);
 	for (i = 0; to && camel_internet_address_get (to, i, &namep, &addressp); i++) {
@@ -391,25 +362,6 @@ camel_mapi_utils_mime_to_item (CamelMimeMessage *message, gint32 message_camel_f
 		mail_item_set_subject(item, camel_mime_message_get_subject(message));
 	}
 
-	headers = camel_medium_get_headers (CAMEL_MEDIUM (message));
-	if (headers) {
-		GString *hstr = g_string_new ("");
-		gint i;
-
-		for (i = 0; i < headers->len; i++) {
-			CamelMediumHeader *h = &g_array_index (headers, CamelMediumHeader, i);
-
-			if (!h->name || !*h->name || g_ascii_strncasecmp (h->name, "X-Evolution", 11) == 0)
-				continue;
-
-			g_string_append_printf (hstr, "%s: %s\n", h->name, h->value ? h->value : "");
-		}
-
-		camel_medium_free_headers (CAMEL_MEDIUM (message), headers);
-
-		item->header.transport_headers = g_string_free (hstr, hstr->len == 0);
-	}
-
 	/*Add message threading properties */
 	item->header.references = g_strdup (camel_medium_get_header ((CamelMedium *) message, "References"));
 	item->header.in_reply_to = g_strdup (camel_medium_get_header ((CamelMedium *) message, "In-Reply-To"));
@@ -420,7 +372,7 @@ camel_mapi_utils_mime_to_item (CamelMimeMessage *message, gint32 message_camel_f
 
 	if (CAMEL_IS_MULTIPART(multipart)) {
 		gboolean is_first = TRUE;
-		if (!mapi_do_multipart (CAMEL_MULTIPART(multipart), item, &is_first))
+		if (!mapi_do_multipart (CAMEL_MULTIPART(multipart), item, &is_first, cancellable))
 			printf("camel message multi part error\n");
 	} else {
 		dw = camel_medium_get_content (CAMEL_MEDIUM (message));
@@ -429,9 +381,10 @@ camel_mapi_utils_mime_to_item (CamelMimeMessage *message, gint32 message_camel_f
 			content_type = camel_content_type_simple (type);
 
 			content_stream = (CamelStream *)camel_stream_mem_new();
-			content_size = camel_data_wrapper_decode_to_stream(dw, (CamelStream *)content_stream, NULL);
+			content_size = camel_data_wrapper_decode_to_stream_sync (
+				dw, (CamelStream *)content_stream, cancellable, NULL);
 
-			mail_item_set_body_stream (item, content_stream, PART_TYPE_PLAIN_TEXT);
+			mail_item_set_body_stream (item, content_stream, PART_TYPE_PLAIN_TEXT, cancellable);
 		}
 	}
 
@@ -447,6 +400,7 @@ camel_mapi_utils_create_item_build_props (ExchangeMapiConnection *conn, mapi_id_
 	MailItem *item = (MailItem *) data;
 	GSList *l;
 	bool send_rich_info;
+	uint32_t msgflag;
 	uint32_t cpid;
 
 	#define set_value(hex, val) G_STMT_START { \
@@ -463,26 +417,8 @@ camel_mapi_utils_create_item_build_props (ExchangeMapiConnection *conn, mapi_id_
 	send_rich_info = false;
 	set_value (PR_SEND_RICH_INFO, &send_rich_info);
 
-	set_value (PR_MESSAGE_FLAGS, &item->header.flags);
-
-	if (item->header.from && *item->header.from)
-		set_value (PR_SENT_REPRESENTING_NAME_UNICODE, item->header.from);
-
-	if (item->header.from_email && *item->header.from_email) {
-		set_value (PR_SENT_REPRESENTING_ADDRTYPE_UNICODE, "SMTP");
-		set_value (PR_SENT_REPRESENTING_EMAIL_ADDRESS_UNICODE, item->header.from_email);
-	}
-
-	if (item->header.recieved_time != 0) {
-		struct FILETIME msg_date = { 0 };
-
-		exchange_mapi_util_time_t_to_filetime (item->header.recieved_time, &msg_date);
-
-		set_value (PR_MESSAGE_DELIVERY_TIME, &msg_date);
-	}
-
-	if (item->header.transport_headers && *item->header.transport_headers)
-		set_value (PR_TRANSPORT_MESSAGE_HEADERS_UNICODE, item->header.transport_headers);
+	msgflag = MSGFLAG_UNSENT;
+	set_value (PR_MESSAGE_FLAGS, &msgflag);
 
 	/* Message threading information */
 	if (item->header.references)
