@@ -135,7 +135,12 @@ fetch_props_to_mail_item_cb (FetchItemsCallbackData *item_data, gpointer data)
 	}
 
 	if (delivery_date) {
-		item->header.recieved_time = exchange_mapi_util_filetime_to_time_t (delivery_date);
+		NTTIME ntdate = { 0 };
+
+		ntdate = delivery_date->dwHighDateTime;
+		ntdate = ntdate << 32;
+		ntdate |= delivery_date->dwLowDateTime;
+		item->header.recieved_time = nt_time_to_unix(ntdate);
 	}
 
 	if (flags && (*flags & MSGFLAG_READ) != 0)
@@ -382,7 +387,7 @@ mapi_mime_set_msg_headers (ExchangeMapiConnection *conn, CamelMimeMessage *msg,
 		camel_mime_parser_scan_from (parser, FALSE);
 		g_object_unref (stream);
 
-		if (camel_mime_part_construct_from_parser (part, parser, NULL) != -1) {
+		if (camel_mime_part_construct_from_parser_sync (part, parser, NULL, NULL)) {
 			struct _camel_header_raw *h;
 
 			for (h = part->headers; h; h = h->next) {
@@ -402,10 +407,6 @@ mapi_mime_set_msg_headers (ExchangeMapiConnection *conn, CamelMimeMessage *msg,
 
 		g_object_unref (parser);
 		g_object_unref (part);
-	} else {
-		recieved_time = item->header.recieved_time;
-		actual_time = camel_header_decode_date (ctime(&recieved_time), &offset);
-		camel_mime_message_set_date (msg, actual_time, offset);
 	}
 
 	/* Overwrite headers if we have specific properties available*/
@@ -413,6 +414,11 @@ mapi_mime_set_msg_headers (ExchangeMapiConnection *conn, CamelMimeMessage *msg,
 	if (temp_str)
 		camel_mime_message_set_subject (msg, temp_str);
 
+	recieved_time = item->header.recieved_time;
+
+	actual_time = camel_header_decode_date (ctime(&recieved_time), &offset);
+	/* camel_mime_message_set_date (msg, actual_time, offset); */
+
 	if (item->header.from) {
 		if ((item->header.from_type != NULL) && !g_utf8_collate (item->header.from_type, "EX")) {
 			gchar *from_email;
@@ -780,7 +786,7 @@ mapi_mime_classify_attachments (ExchangeMapiConnection *conn, mapi_id_t fid, con
 			CamelStream *mem;
 
 			mem = camel_stream_mem_new ();
-			camel_stream_write (mem, (const gchar *) stream->value->data, stream->value->len, NULL);
+			camel_stream_write (mem, (const gchar *) stream->value->data, stream->value->len, NULL, NULL);
 			camel_stream_reset (mem, NULL);
 
 			parser = camel_mime_parser_new ();
@@ -794,7 +800,7 @@ mapi_mime_classify_attachments (ExchangeMapiConnection *conn, mapi_id_t fid, con
 				part = camel_mime_part_new ();
 
 				camel_data_wrapper_set_mime_type_field (CAMEL_DATA_WRAPPER (part), camel_mime_parser_content_type (parser));
-				camel_mime_part_construct_content_from_parser (part, parser, NULL);
+				camel_mime_part_construct_content_from_parser (part, parser, NULL, NULL);
 			} else {
 				is_smime = FALSE;
 			}
